#version 330

// Task 1: add input variable of type vec3 for vNormal
// Task 4: add input variable of type vec3 for vPosition
in vec3 vNormal;
in vec3 vPosition;

out vec4 fragColor;

// Task 3: add uniform variables for light properties
//           use vec4 for lightPosition
//           use vec3 for lightAmbientColor and lightDiffuseColor
// Task 4: add lightSpecularColor of type vec3
uniform vec4 lightPosition;
uniform vec3 lightAmbientColor;
uniform vec3 lightDiffuseColor;
uniform vec3 lightSpecularColor;

// Task 4: add vec3 eyePosition uniform variable to store position
uniform vec3 eyePosition;

// Task 5: add uniform variables for material properties
//           use vec3 type for materialAmbientColor, materialDiffuseColor and materialSpecularColor
//           use float type for materialShininess
uniform vec3 materialAmbientColor;
uniform vec3 materialDiffuseColor;
uniform vec3 materialSpecularColor;
uniform float materialShininess;

void main() {
    // Task 1: add vec3 lightDirection variable to store normalized light direction, you can use (0,1,0) for now
    // Task 3: set the lightDirection to normalized light position fom the uniform variable (ignore the 4th component for now)
    // Task 8: modify the lightDirection calculation;
    //           for directional light (w == 0), keep it unchanged
    //           for point lights (w != 0), it's direction from vertex position to light
    vec3 lightDirection;
    if(lightPosition.w == 0.0) {
        lightDirection = normalize(lightPosition.xyz);
    } else {
        lightDirection = normalize(lightPosition.xyz - vPosition);
    }

    // Task 4: add vec3 viewDirection variable to store direction from vertex position to eye position
    //           don't forget to normalize it
    //         add vec3 halfVector variable to calculate the halfway vector as normalize(lightDirection + viewDirection)
    vec3 viewDirection = normalize(eyePosition - vPosition);
    vec3 halfVector = normalize(lightDirection + viewDirection);

    // Task 1: create float diffuseFactor and vec3 diffuse variable to calculate the diffuse light
    //           calculate the diffuse factor as max(dot(vNormal, lightDirection), 0)
    //           calculate the diffuse light by multiplying light color by diffuseFactor
    //             use hardcoded white light (1, 1, 1) as a light color for now
    // Task 3: change light color to uniform lightDiffuseColor
    // Task 5: multiply the diffuse variable by materialDiffuseColor
    float diffuseFactor = max(dot(vNormal, lightDirection), 0.0);
    vec3 diffuse = lightDiffuseColor * materialDiffuseColor * diffuseFactor;

    // Task 2: create ambient variable to calculate the ambient light
    //           set the ambient color to dark grey, ie. vec3(0.15)
    // Task 3: change light color to uniform lightAmbientColor
    // Task 5: multiply the ambient variable by materialAmbientColor
    vec3 ambient = lightAmbientColor * materialAmbientColor;

    // Task 4: create specular variable and calculate the specular light
    //           the calculation is lightSpecularColor * pow(max(dot(vNormal, halfVector), 0), 32) * diffuseFactor
    // Task 5: multiply the specular variable by materialSpecularColor
    //           change the exponent from a constant number to materialShininess
    vec3 specular = lightSpecularColor * materialSpecularColor * pow(max(dot(vNormal, halfVector), 0.0), materialShininess) * diffuseFactor;

    // Task 1: assign the calculated diffuse variable to lightFinal
    // Task 2: add the ambient variable to lightFinal calculation
    // Task 4: add the specular variable to lightFinal calculation
    vec3 lightFinal = ambient + diffuse + specular;

    fragColor = vec4(lightFinal, 1.0);
}